'   Subject: Displacement Map Demo
'    Author: Liquidex
'     Email: LordQBASIC@aol.com
'   Version: 1.0
'      Date: 1.18.2000
' Copyright: (C) 2000 Alex Varanese of The Alphx Coding Group

DECLARE SUB PrintLogo ()
DECLARE SUB InitTextMode ()
DECLARE SUB GenInit ()
DECLARE SUB DrawPoint (H%, V%, Col%, DstSeg%, DstOff%)
DECLARE SUB DrawLine (H1%, V1%, H2%, V2%, Col%, DstSeg%, DstOff%)
DECLARE SUB LoadPal (Filename$, PalSeg%, PalOff%)
DECLARE SUB ShutDown ()
DECLARE SUB InitGenData ()
DECLARE SUB ClrKBBuffer ()
DECLARE SUB ErrExit (ErrStr$)
DECLARE SUB BufferWaitAnyKey ()
DECLARE SUB SetPal (PalSeg%, PalOff%)
DECLARE FUNCTION GetPoint% (H%, V%, SrcSeg%, SrcOff%)
DECLARE SUB CreateMap ()

ON ERROR GOTO ErrHandler

DEFINT A-Z
'$DYNAMIC

CONST Pi = 3.14159265359#
CONST RadianCnvrtRatio = Pi / 180

CONST ScrnHRes& = 320, ScrnVRes& = 200
CONST HalfScrnH = ScrnHRes / 2 - 1, HalfScrnV = ScrnVRes / 2 - 1
CONST MaxScrnH = ScrnHRes - 1, MaxScrnV = ScrnVRes - 1
CONST FrameBufferByteSize& = ScrnHRes& * ScrnVRes&
CONST FrameBufferWordSize& = FrameBufferByteSize& / 2

CONST MapHVrtxSize = 32, MapVVrtxSize = 32
CONST MapVrtxScalar = 8, MapDispHeightScalar = 8
CONST MapHVrtxDensity = 4, MapVVrtxDensity = 4
CONST MapBlurDepth = 2

CONST LensHPxlSize& = 256

TYPE Rect2DType
 H1                             AS INTEGER
 V1                             AS INTEGER
 H2                             AS INTEGER
 V2                             AS INTEGER
END TYPE

TYPE Vrtx3DType
 X                              AS SINGLE
 Y                              AS SINGLE
 Z                              AS SINGLE
 Col                            AS INTEGER
END TYPE

DIM SHARED VOffMap(MaxScrnV) AS LONG
DIM SHARED FrameBuffer(FrameBufferWordSize + 1)
DIM SHARED ClipRgn AS Rect2DType
DIM SHARED DefPal(383)

DIM SHARED Map(MapHVrtxSize, MapVVrtxSize) AS Vrtx3DType

DIM SHARED FPS

GenInit

CamDist = 256

XAng! = 0
XCos! = COS(XAng!)
XSin! = SIN(XAng!)
YAng! = 4.5 * RadianCnvrtRatio
YCos! = COS(YAng!)
YSin! = SIN(YAng!)
ZAng! = 1.5 * RadianCnvrtRatio
ZCos! = COS(ZAng!)
ZSin! = SIN(ZAng!)

SecTimer! = TIMER + 1
FPS = 0

DO
 FOR V = 1 TO MapVVrtxSize - 2
  FOR H = 1 TO MapHVrtxSize - 2
   Z! = (CamDist - Map(H, V).Z)
   X1 = (LensHPxlSize& * Map(H, V).X) / Z! + HalfScrnH
   Y1 = HalfScrnV - ((LensHPxlSize& * Map(H, V).Y) / Z!)
   Z! = (CamDist - Map(H + 1, V).Z)
   X2 = ((LensHPxlSize& * Map(H + 1, V).X) / Z!) + HalfScrnH
   Y2 = HalfScrnV - ((LensHPxlSize& * Map(H + 1, V).Y) / Z!)
   Z! = (CamDist - Map(H, V + 1).Z)
   X3 = ((LensHPxlSize& * Map(H, V + 1).X) / Z!) + HalfScrnH
   Y3 = HalfScrnV - ((LensHPxlSize& * Map(H, V + 1).Y) / Z!)
   DrawLine X1, Y1, X2, Y2, Map(H, V).Col, VARSEG(FrameBuffer(0)), VARPTR(FrameBuffer(2))
   DrawLine X1, Y1, X3, Y3, Map(H, V).Col, VARSEG(FrameBuffer(0)), VARPTR(FrameBuffer(2))
  NEXT H
 NEXT V

 FOR H = 1 TO MapHVrtxSize
  FOR V = 1 TO MapVVrtxSize
   Y! = Map(H, V).Y * XCos! - Map(H, V).Z * XSin!
   Z! = Map(H, V).Z * XCos! + Map(H, V).Y * XSin!
   Map(H, V).Y = Y!
   Map(H, V).Z = Z!
   Z! = Map(H, V).Z * YCos! - Map(H, V).X * YAng!
   X! = Map(H, V).X * YCos! + Map(H, V).Z * YAng!
   Map(H, V).Z = Z!
   Map(H, V).X = X!
   X! = Map(H, V).X * ZCos! - Map(H, V).Y * ZSin!
   Y! = Map(H, V).Y * ZCos! + Map(H, V).X * ZSin!
   Map(H, V).X = X!
   Map(H, V).Y = Y!
  NEXT V
 NEXT H

 PUT (0, 0), FrameBuffer, PSET
 REDIM SHARED FrameBuffer(FrameBufferWordSize + 1)
 FrameBuffer(0) = 2560
 FrameBuffer(1) = 200

 IF TIMER <= SecTimer! THEN
  FPS = FPS + 1
 ELSEIF INKEY$ <> "" THEN EXIT DO
 END IF
LOOP

ShutDown

ErrHandler:
 InitTextMode
 SELECT CASE ERR
  CASE 6: ErrStr$ = "Overflow"
  CASE 7: ErrStr$ = "Out of memory"
  CASE 9: ErrStr$ = "Subscript out of range"
  CASE 11: ErrStr$ = "Division by zero"
  CASE 14: ErrStr$ = "Out of string space"
  CASE 28: ErrStr$ = "Out of stack space"
  CASE 51: ErrStr$ = "Internal error"
  CASE 52: ErrStr$ = "Bad file name/number"
  CASE 53: ErrStr$ = "File not found"
  CASE 54: ErrStr$ = "Bad file mode"
  CASE 62: ErrStr$ = "Data not found"
  CASE 76: ErrStr$ = "Path not found"
  CASE ELSE: ErrStr$ = "Unkown error"
 END SELECT
 ErrExit ErrStr$

REM $STATIC
SUB BufferWaitAnyKey
 ClrKBBuffer
 DO: LOOP UNTIL INKEY$ <> ""
END SUB

SUB ClrKBBuffer
 FOR I = 1 TO 10
  CurKeyASCII$ = INKEY$
 NEXT I
END SUB

SUB CreateMap
 FOR H1 = 1 TO MapHVrtxSize - 1 STEP MapHVrtxDensity
  FOR V1 = 1 TO MapVVrtxSize - 1 STEP MapVVrtxDensity
   Col = RND * 255
   FOR H2 = H1 TO H1 + MapHVrtxDensity - 1
    FOR V2 = V1 TO V1 + MapVVrtxDensity - 1
     IF H2 < MapHVrtxSize AND V2 < MapVVrtxSize THEN Map(H2, V2).Col = Col
    NEXT V2
   NEXT H2
  NEXT V1
 NEXT H1

 FOR CurBlur = 1 TO MapBlurDepth
  FOR H = 1 TO MapHVrtxSize - 1
   FOR V = 1 TO MapVVrtxSize - 1
    Col = Map(H, V).Col
    Col = Col + Map(H + 1, V).Col
    Col = Col + Map(H - 1, V).Col
    Col = Col + Map(H, V + 1).Col
    Col = Col + Map(H, V - 1).Col
    Col = Col / 5
    Map(H, V).Col = Col
   NEXT V
  NEXT H
 NEXT CurBlur

 X = 0 - (MapHVrtxSize / 2) * MapVrtxScalar
 FOR H = 1 TO MapHVrtxSize
  Z = 0 - (MapVVrtxSize / 2) * MapVrtxScalar
  FOR V = 1 TO MapVVrtxSize
   Map(H, V).Col = -(255 - Map(H, V).Col)
   Map(H, V).X = X
   Map(H, V).Y = (Map(H, V).Col / MapDispHeightScalar) + 56
   Map(H, V).Z = Z
   Z = Z + MapVrtxScalar
  NEXT V
  X = X + MapVrtxScalar
 NEXT H
END SUB

SUB DrawLine (H1, V1, H2, V2, Col, DstSeg, DstOff)
 H = H1
 V = V1

 DeltaH = H2 - H1
 DeltaV = V2 - V1
 IF DeltaH < 0 THEN DeltaH = -DeltaH
 IF DeltaV < 0 THEN DeltaV = -DeltaV

 HInc = 1
 VInc = 1
 IF H1 > H2 THEN HInc = -1
 IF V1 > V2 THEN VInc = -1

 IF DeltaH >= DeltaV THEN
  PxlNum = DeltaH
  DeltaV2 = DeltaV * 2
  ErrTerm = (DeltaV2) - DeltaH
  ErrTermInc1 = DeltaV2
  ErrTermInc2 = (DeltaV - DeltaH) * 2

  DO
   DrawPoint H, V, Col, DstSeg, DstOff
   IF ErrTerm < 0 THEN
    ErrTerm = ErrTerm + ErrTermInc1
   ELSE
    ErrTerm = ErrTerm + ErrTermInc2
    V = V + VInc
   END IF
   H = H + HInc
   PxlNum = PxlNum - 1
  LOOP UNTIL PxlNum < 0
 ELSE
  PxlNum = DeltaV
  DeltaH2 = DeltaH * 2
  ErrTerm = (DeltaH2) - DeltaV
  ErrTermInc1 = DeltaH2
  ErrTermInc2 = (DeltaH - DeltaV) * 2

  DO
   DrawPoint H, V, Col, DstSeg, DstOff
   IF ErrTerm < 0 THEN
    ErrTerm = ErrTerm + ErrTermInc1
   ELSE
    ErrTerm = ErrTerm + ErrTermInc2
    H = H + HInc
   END IF
   V = V + VInc
   PxlNum = PxlNum - 1
  LOOP UNTIL PxlNum < 0
 END IF
END SUB

SUB DrawPoint (H, V, Col, DstSeg, DstOff)
 IF H >= ClipRgn.H1 AND H <= ClipRgn.H2 AND V >= ClipRgn.V1 AND V <= ClipRgn.V2 THEN
  DEF SEG = DstSeg
  POKE (VOffMap(V) + H + DstOff), Col
  DEF SEG
 END IF
END SUB

SUB ErrExit (ErrStr$)
 PRINT
 PRINT "Error - " + ErrStr$ + "."
 PRINT "Press any key to exit..."
 BufferWaitAnyKey
 CLS
 END
END SUB

SUB GenInit
 InitTextMode
 CLS

 PrintLogo
 PRINT
 InitGenData
 PRINT
 PRINT "Press any key to begin..."
 BufferWaitAnyKey

 SCREEN 13
 LoadPal "Liquid3D.PAL", VARSEG(DefPal(0)), VARPTR(DefPal(0))
 SetPal VARSEG(DefPal(0)), VARPTR(DefPal(0))
END SUB

FUNCTION GetPoint (H, V, SrcSeg, SrcOff)
 DEF SEG = SrcSeg
 GetPoint = PEEK(VOffMap(V) + H + SrcOff)
 DEF SEG
END FUNCTION

SUB InitGenData
 PRINT "- Calculating vertical offset map... ";
 FOR V = 0 TO MaxScrnV
  VOffMap(V) = V * ScrnHRes
 NEXT V
 PRINT "done."

 PRINT "- Initializing misc. engine data... ";
 RANDOMIZE TIMER
 FrameBuffer(0) = 2560
 FrameBuffer(1) = 200
 ClipRgn.H1 = 0
 ClipRgn.V1 = 0
 ClipRgn.H2 = MaxScrnH
 ClipRgn.V2 = MaxScrnV
 PRINT "done."

 PRINT "- Generating terrain... ";
 CreateMap
 PRINT "done."
END SUB

SUB InitTextMode
 SCREEN 0
 WIDTH 80, 25
END SUB

SUB LoadPal (Filename$, PalSeg, PalOff)
 DEF SEG = PalSeg
 TempPalOff = PalOff
 BLOAD Filename$, TempPalOff
 DEF SEG
END SUB

SUB PrintLogo
 PRINT "3D Displacement Map Demo v1.0"
 PRINT "Copyright (C) 2000 The Alphx Coding Group"
 PRINT "Written by Alex Varanese"
 PRINT STRING$(80, "")
END SUB

SUB SetPal (PalSeg, PalOff)
 DEF SEG = PalSeg
 CurPalOff = PalOff
 FOR CurCol = 0 TO 255
  OUT &H3C8, CurCol
  OUT &H3C9, PEEK(CurPalOff)
  OUT &H3C9, PEEK(CurPalOff + 1)
  OUT &H3C9, PEEK(CurPalOff + 2)
  CurPalOff = CurPalOff + 3
 NEXT CurCol
 DEF SEG
END SUB

SUB ShutDown
 InitTextMode
 PrintLogo
 PRINT
 PRINT "Thanks for trying out this demo!"
 PRINT
 PRINT "FPS: "; FPS
 END
END SUB

